#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "networktab.h"
#include "controllertab.h"
#include "perturbationtab.h"
#include "motortab.h"
#include "communicationtab.h"
#include "tinyxml2.h"
#include "qtsecosocket.h"

#include <fstream>
#include <iostream>
#include <string>

using namespace std;
using namespace tinyxml2;

class QAction;
class QActionGroup;
class QLabel;
class QMenu;

class MainWindow : public QMainWindow
{
	Q_OBJECT

	public:
		MainWindow(PlotWindow *q_plot, NetworkTab *network_tab, ControllerTab *controller_tab, PerturbationTab *perturbation_tab, MotorTab *motor_tab, CommunicationTab *communication_tab);

	protected:
		void contextMenuEvent(QContextMenuEvent *event);

		private slots:
		void open();
		void save();
		void savePdf();
		void loadPlots();
		
		void newFile();
		void print();
		void undo();
		void redo();
		void cut();
		void copy();
		void paste();
		void bold();
		void italic();
		void leftAlign();
		void rightAlign();
		void justify();
		void center();
		void setLineSpacing();
		void setParagraphSpacing();
		void about();
		void aboutQt();

	private:
		void createActions();
		void createMenus();

		QMenu *fileMenu;
		QMenu *editMenu;
		QMenu *formatMenu;
		QMenu *helpMenu;
		QActionGroup *alignmentGroup;
		QAction *openAct;
		QAction *saveAct;
		
		QAction *savePdfAct;
		QAction *loadPlotAct;
		
		QAction *newAct;
		QAction *printAct;
		QAction *exitAct;
		QAction *undoAct;
		QAction *redoAct;
		QAction *cutAct;
		QAction *copyAct;
		QAction *pasteAct;
		QAction *boldAct;
		QAction *italicAct;
		QAction *leftAlignAct;
		QAction *rightAlignAct;
		QAction *justifyAct;
		QAction *centerAct;
		QAction *setLineSpacingAct;
		QAction *setParagraphSpacingAct;
		QAction *aboutAct;
		QAction *aboutQtAct;
		QLabel *infoLabel;

		NetworkTab 				*networkTab;
		ControllerTab 		*controllerTab;
		PerturbationTab 	*perturbationTab;
		MotorTab 					*motorTab;
		CommunicationTab 	*communicationTab;
		PlotWindow				*plot;
};

#endif
