#ifndef TABDIALOG_H
#define TABDIALOG_H

#include <QDialog>
#include <QtGui>
#include <QLocale>

//For reimplementing printf
#include <stdio.h>
#include <stdarg.h>

#include "networktab.h"
#include "controllertab.h"
#include "perturbationtab.h"
#include "motortab.h"
#include "communicationtab.h"
#include "communicationdatatab.h"

#include "qtsecosocket.h"
#include "qlogtext.h"

#include "plotwindow.h"
#include "camerawindow.h"
#include "qledindicator.h"

#include "mainwindow.h"

/*class QDialogButtonBox;*/
class QFileInfo;
class QTabWidget;

class TabDialog : public QDialog
{
	Q_OBJECT

	public:
		TabDialog(PlotWindow *q_plot, CameraWindow *q_camera, QWidget *parent = 0);
    void SetLedOn   ( void );
    void SetLedOff  ( void );
    void SetQueue   ( int value );
	private:
		void createBottomBar 	( void );
		
		QTabWidget 	*tabWidget;
		QGroupBox 	*bottomBarGroup;
		
		NetworkTab 						*networkTab;
		MotorTab							*motorTab;
		ControllerTab					*controllerTab;
		PerturbationTab				*perturbationTab;
		CommunicationTab 			*communicationTab;
		CommunicationDataTab 	*communicationDataTab;
		QtSecoSocket					*qtsecosocket;

		QComboBox							*referenceType;
		QDoubleSpinBox				*referenceValue1;
		QComboBox							*controlVar;

		QPushButton 					*printButton;
    QPushButton 					*connectButton;
		QPushButton 					*disconnectButton;
    QPushButton 					*runButton;
		QPushButton 					*resetButton;
    QLedIndicator         *led1;
    QLedIndicator         *led2;
    QLedIndicator         *led3;
    QLedIndicator         *led4;
    QLineEdit             *queueValue;


		PlotWindow						*plot;
		CameraWindow					*camera;
	
		MainWindow 						*window;
		
		QTimer 								*m_TCheckSocketTimer;
		QTimer 								*m_TCheckDataEndTimer;
		/*unsigned char kk;*/
	private slots:
		void printData 					( void );
	
		void SocketConnect 			( void );
		void SocketDisconnect 	( void );
		void SocketRun			 		( void );
		void SocketReset 				( void );
		void configPlot					( void );

		void correctQuit 				( void );
		void checkSocketStatus 	( void );
		void checkDataEnd 			( void );
};

#endif
