#include "motortab.h"

/****************************************
 * **************************************/
MotorTab::MotorTab( void ) 
{
	/* Create Main Layout */
	//QVBoxLayout *mainLayout = new QVBoxLayout;
	QGridLayout *mainLayout = new QGridLayout;

	/* For the number of motors */
	for ( int i = 0 ; i < NUM_MOTORS ; i++)
	{
		/* Create Motor group */
		QGroupBox *motorGroup = new QGroupBox(tr("Motor %1").arg(i+1));
		
		/* Create motor layout */
		QFormLayout *motorLayout = new QFormLayout;
		
		/* Create motor Flag On/Off */
		motorOn[i] = new QCheckBox();
		
		/* Config motor Flag On/Off, On only for Motor3 */
		if ( i == 2 ) //Motor3
			motorOn[i]->setChecked(true);
		else
			motorOn[i]->setChecked(false);
		/* Disabel all checks */
		motorOn[i]->setEnabled(false);
		
		/* Create motor PWM Freq values */
		motorPWMFreq[i]	= new QSpinBox;
		/* Config motor PWM Freq values */
		motorPWMFreq[i]->setRange(20000, 40000);
		motorPWMFreq[i]->setSingleStep(1);
		motorPWMFreq[i]->setValue(20000);
		motorPWMFreq[i]->setSuffix(" Hz");
		motorPWMFreq[i]->setFixedWidth(100); 
	
		/* Enable only if checked */
		if (motorOn[i]->isChecked())
			motorPWMFreq[i]->setEnabled(false);
		else
			motorPWMFreq[i]->setEnabled(false);

		/* Add data to motor layout */
		motorLayout->addRow(new QLabel(tr("On/Off:   ")),motorOn[i]);
		motorLayout->addRow(new QLabel(tr("PWM Freq.:")),motorPWMFreq[i]);

		/* Add layout to motor group */
		motorGroup->setLayout(motorLayout);

		div_t d;
		d = div (i, 2);
		/* Add motor group to main layout */
		mainLayout->addWidget(motorGroup, d.quot, d.rem);
	}
	/* Create a new blank row with a big stretch to allow 
	 * the real text to be on top */
	mainLayout->setRowStretch(NUM_MOTORS/2, 2);
	setLayout(mainLayout);
}

/****************************************
 ***************************************/

unsigned char MotorTab::getMotorStatus ( unsigned char u_motor ) 
{
	return motorOn[u_motor]->isChecked();
}

/****************************************
 ***************************************/

unsigned int MotorTab::getMotorPWMFreq ( unsigned char u_motor ) 
{
	return motorPWMFreq[u_motor]->value();
}

/****************************************
 ***************************************/

unsigned char MotorTab::getNumMotors ( void )
{
	return NUM_MOTORS;
}

/****************************************
 ***************************************/

void  MotorTab::enableMotor 			( unsigned char u_motor )
{
	motorOn[u_motor]->setEnabled(true);
	motorOn[u_motor]->setChecked(true);
	motorPWMFreq[u_motor]->setEnabled(true);
}

/****************************************
 ***************************************/

void 	MotorTab::disableMotor			( unsigned char u_motor )
{
	motorOn[u_motor]->setChecked(false);
	motorOn[u_motor]->setEnabled(false);
	motorPWMFreq[u_motor]->setEnabled(false);
}

/****************************************
 ***************************************/

void  MotorTab::setMotorPWMFreq		( unsigned char u_motor, unsigned int u_freq )
{
	motorPWMFreq[u_motor]->setValue(u_freq);
}

/****************************************
 ***************************************/
