/***************************************************************************
**                                                                        **
**  QCustomPlot, an easy to use, modern plotting widget for Qt            **
**  Copyright (C) 2011, 2012, 2013 Emanuel Eichhammer                     **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program.  If not, see http://www.gnu.org/licenses/.   **
**                                                                        **
****************************************************************************
**           Author: Emanuel Eichhammer                                   **
**  Website/Contact: http://www.qcustomplot.com/                          **
**             Date: 05.09.13                                             **
**          Version: 1.0.1                                                **
****************************************************************************/

#include "plotwindow.h"
#include "ui_plotwindow.h"
#include <iostream>

const char* colorTable[VARNAMES] = {"black", "red", "blue", "green", "yellow", "magenta", "gray", "cyan", "darkBlue", "darkRed", "darkCyan", "darkMagenta", "darkGreen"};
PlotWindow::PlotWindow(QWidget *parent) :
  QMainWindow(parent),
  ui(new Ui::PlotWindow)
{
  ui->setupUi(this);
	//setupPlot();
  
  // configure scroll bars:
  // Since scroll bars only support integer values, we'll set a high default range of -500..500 and
  // divide scroll bar position values by 100 to provide a scroll range -5..5 in floating point
  // axis coordinates. if you want to dynamically grow the range accessible with the scroll bar,
  // just increase the the minimum/maximum values of the scroll bars as needed.
	//ui->horizontalScrollBar->setRange(-200, 200);
	//ui->verticalScrollBar->setRange(-3000, 3000);
  
  // create connection between axes and scroll bars:
	//connect(ui->horizontalScrollBar, SIGNAL(valueChanged(int)), this, SLOT(horzScrollBarChanged(int)));
	//connect(ui->verticalScrollBar, SIGNAL(valueChanged(int)), this, SLOT(vertScrollBarChanged(int)));
	//connect(ui->plot->xAxis, SIGNAL(rangeChanged(QCPRange)), this, SLOT(xAxisChanged(QCPRange)));
	//connect(ui->plot->yAxis, SIGNAL(rangeChanged(QCPRange)), this, SLOT(yAxisChanged(QCPRange)));
  
  // initialize axis range (and scroll bar positions via signals we just connected):
	//ui->plot->xAxis->setRange(1, 2, Qt::AlignCenter);
	//ui->plot->yAxis->setRange(15, 35, Qt::AlignCenter);
	ui->plot->xAxis->setRange(0, 1);
	ui->plot->yAxis->setRange(0, 1);

	fileNameVar[POS]           ="POS";
	fileNameVar[SPEED]         ="VEL";
	fileNameVar[CURRENT]       ="CUR";
	fileNameVar[REF]           ="REF";
	fileNameVar[ERROR]         ="ERR";
	fileNameVar[ERROR_REF_FB]  ="ERB";
	fileNameVar[UFF]           ="UFF";
	fileNameVar[UDI]           ="UDI";
	fileNameVar[UFB]           ="UFB";
	fileNameVar[UPA]           ="UPA";
	fileNameVar[UALL]          ="U";
	fileNameVar[USAT]          ="USAT";
	fileNameVar[Y]          ="Y";
	
	printNameVar[POS]           ="POS (rad)";
	printNameVar[SPEED]         ="VEL (rad/s)";
	printNameVar[CURRENT]       ="CUR (A)";
	printNameVar[REF]           ="REF";
	printNameVar[ERROR]         ="ERR";
	printNameVar[ERROR_REF_FB]  ="ERB";
	printNameVar[UFF]           ="UFF (V)";
	printNameVar[UDI]           ="UDI (V)";
	printNameVar[UFB]           ="UFB (V)";
	printNameVar[UPA]           ="UPA (V)";
	printNameVar[UALL]          ="U (V)";
	printNameVar[USAT]          ="USAT (V)";
	printNameVar[Y]          ="Y";

	uPlotFlag = FALSE;
}

void PlotWindow::setPrintNames (unsigned int var)
{
		switch(var)
		{
			case POS:
				printNameVar[REF] 					= "REF (rad)";
				printNameVar[ERROR] 				= "ERR (rad)";
				printNameVar[ERROR_REF_FB] 	= "ERB (rad)";
				printNameVar[Y] 						= "Y   (rad)";
				break;
			case SPEED:
				printNameVar[REF] 					= "REF (rad/s)";
				printNameVar[ERROR] 				= "ERR (rad/s)";
				printNameVar[ERROR_REF_FB] 	= "ERB (rad/s)";
				printNameVar[Y] 						= "Y   (rad/s)";
				break;
		}
}

PlotWindow::~PlotWindow()
{
  delete ui;
}

void PlotWindow::setupPlot()
{

	globalCustomPlot=ui->plot;
	unsigned int graphCounter=0;
	ui->plot->clearGraphs();
	for (int i = 0 ; i < VARNAMES ; i++)
	{
		if (motorTime[i].size() != 0 )
		{
			ui->plot->addGraph();
			QPen graphPen;
			QColor color;
			//graphPen.setColor(QColor(rand()%245+10, rand()%245+10, rand()%245+10));
			graphPen.setColor(QColor(colorTable[i]));
			graphPen.setWidth(2);
			ui->plot->graph(graphCounter)->setPen(graphPen);
			ui->plot->graph(graphCounter)->setData(motorTime[i], motorValue[i]);
			ui->plot->graph()->setName(printNameVar[i]);

			if (graphCounter==0)
				ui->plot->graph(graphCounter)->rescaleAxes();                                                                  
			else
				ui->plot->graph(graphCounter)->rescaleAxes(true);  
			graphCounter++;
		}
	}
	// configure right and top axis to show ticks but no labels:                                           
	//   // (see QCPAxisRect::setupFullAxesBox for a quicker method to do this)    
	ui->plot->xAxis2->setVisible(true);                                                                  
	ui->plot->xAxis2->setTickLabels(false);                                                              
	ui->plot->yAxis2->setVisible(true);                                                                  
	ui->plot->yAxis2->setTickLabels(false);

	connect(ui->plot->xAxis, SIGNAL(rangeChanged(QCPRange)), ui->plot->xAxis2, SLOT(setRange(QCPRange)), Qt::UniqueConnection);
	connect(ui->plot->yAxis, SIGNAL(rangeChanged(QCPRange)), ui->plot->yAxis2, SLOT(setRange(QCPRange)), Qt::UniqueConnection); 
 	connect(ui->plot, SIGNAL(mousePress(QMouseEvent*)), this, SLOT(clickedGraph(QMouseEvent*)), Qt::UniqueConnection);
 	connect(ui->plot, SIGNAL(mouseRelease(QMouseEvent*)), this, SLOT(mouseReleaseEvent(QMouseEvent*)), Qt::UniqueConnection);

	// let the ranges scale themselves so graph 0 fits perfectly in the visible area:                      
	//ui->plot->graph(0)->rescaleAxes();                                                                   
	//     // same thing for graph 1, but only enlarge ranges (in case graph 1 is smaller than graph 0):          
	//ui->plot->graph(1)->rescaleAxes(true);             
	//ui->plot->axisRect()->setupFullAxesBox(true);
	ui->plot->legend->setVisible(true);
	ui->plot->legend->setFont(QFont("Helvetica", 15));

	//ui->plot->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom);
	ui->plot->setInteractions(QCP::iRangeZoom);
	ui->plot->replot();

	defaultXCoordA = globalCustomPlot->xAxis->range().lower;
	defaultXCoordB = globalCustomPlot->xAxis->range().upper;
	defaultYCoordA = globalCustomPlot->yAxis->range().lower;
	defaultYCoordB = globalCustomPlot->yAxis->range().upper;
  
	statusBar()->clearMessage();
	globalCustomPlot->replot();
	
	//printf("TIME LIMIT: %2.2f\n", fPlotMaxTime);
	
}

void PlotWindow::keyPressEvent( QKeyEvent * event )
{
	if( event->key() == Qt::Key_A )
	{
		//printf("IN KEY PRES A\n");
		globalCustomPlot->xAxis->setRange(defaultXCoordA, defaultXCoordB);
		globalCustomPlot->yAxis->setRange(defaultYCoordA, defaultYCoordB);
		globalCustomPlot->replot();
	}
}

void PlotWindow::mouseReleaseEvent ( QMouseEvent * mouseEvent )
{
	
	if ( mouseEvent->button() == Qt::LeftButton )
	{
		newXCoordB = globalCustomPlot->xAxis->pixelToCoord(mouseEvent->pos().x());
		newYCoordB = globalCustomPlot->yAxis->pixelToCoord(mouseEvent->pos().y());
		
		//printf("newXCoordA: %2f, newYCoordA: %2f\n", newXCoordA, newYCoordA);
		//printf("newXCoordB: %2f, newYCoordB: %2f\n", newXCoordB, newYCoordB);
		globalCustomPlot->xAxis->setRange(newXCoordA, newXCoordB);
		globalCustomPlot->yAxis->setRange(newYCoordA, newYCoordB);
		globalCustomPlot->replot();
	}
}; 

void PlotWindow::clickedGraph(QMouseEvent *mouseEvent)
{
	if ( mouseEvent->button() == Qt::LeftButton )
	{
		newXCoordA = globalCustomPlot->xAxis->pixelToCoord(mouseEvent->pos().x());
		newYCoordA = globalCustomPlot->yAxis->pixelToCoord(mouseEvent->pos().y());

		//printf("newXCoordA: %2f, newYCoordA: %2f\n", newXCoordA, newYCoordA);
	}

	if ( mouseEvent->button() == Qt::RightButton )
	{
		// QPoint p = this->mapFromGlobal(QCursor::pos()); //get mouse position
		//QPoint  p = mouseEvent->pos();
		//printf("X: %d, Y: %d\n", p.x(),p.y());
		double xCoord = globalCustomPlot->xAxis->pixelToCoord(mouseEvent->pos().x());
		double yCoord = globalCustomPlot->yAxis->pixelToCoord(mouseEvent->pos().y());
		//qDebug() << globalCustomPlot->xAxis->pixelToCoord(mouseEvent->pos().x()) << globalCustomPlot->yAxis->pixelToCoord(mouseEvent->pos().y());

		int itemNumber = globalCustomPlot->itemCount();
		//printf("NUMBER of ITEMS: %d\n", itemNumber);
		if ( itemNumber != 0)
			globalCustomPlot->removeItem(itemNumber-1);

		QCPItemText *textLabel = new QCPItemText(globalCustomPlot);
		globalCustomPlot->addItem(textLabel);
		textLabel->setPositionAlignment(Qt::AlignTop|Qt::AlignHCenter);
		textLabel->position->setType(QCPItemPosition::ptAxisRectRatio);
		textLabel->position->setCoords(0.5, 0.9); // place position at center/top of axis rect
		//textLabel->position->setCoords(p.x(), p.y()); // place position at center/top of axis rect
		//textLabel->position->setCoords(xCoord, yCoord); // place position at center/top of axis rect
		char coordsString[1024];
		sprintf(coordsString,"X: %2.4f -- Y: %2.4f\n", xCoord, yCoord);
		//printf("%s\n", coordsString);
		textLabel->setText(coordsString);
		textLabel->setFont(QFont(font().family(), 16)); // make font a bit larger
		textLabel->setPen(QPen(Qt::black)); // show black border around text

		globalCustomPlot->replot();
	}
}

void PlotWindow::addValue (unsigned char u_motor, unsigned char var, float f_time, float f_value )
{
	//printf("addValue\n");
	if ( f_time <= fPlotMaxTime ) 
	{
		//printf("A -- Var: %d\n", var);
		if ( commVarFlag[var] == true )
		{
			//printf("B -- Time: %2f, Value: %2f\n", f_time, f_value);
			uPlotFlag = TRUE;
			motorTime[var].push_back(f_time);
			motorValue[var].push_back(f_value);
		}
	}
	else if (uPlotFlag == TRUE)
	{
		//printf("C\n");
		//fflush(stdout);
		setupPlot();
		uPlotFlag = FALSE;
	}
}

void PlotWindow::setTimeLimit ( float f_time )
{	
	fPlotMaxTime = f_time;
}

void PlotWindow::clearData ( void )
{	
	//Clear Vector
	for (int i = 0 ; i < VARNAMES ; i++)
	{
		motorTime[i].clear();
		motorValue[i].clear();
	}
}

void PlotWindow::setCommVarFlag ( unsigned char var, unsigned char value )
{
	commVarFlag[var] = value;	
}
void PlotWindow::savePdf (QString filename)
{
	globalCustomPlot->savePdf(filename, false, 800, 600);
}

/***************************************
 * ************************************/

void PlotWindow::loadPlotsFromSocket(string s_path)
{
	/* Clear plot data */
	clearData();
	setTimeLimit(1000);

	unsigned int uIndexMotor  = 0;                                                                       
	unsigned int uIndexVar    = 0; 

  unsigned int spanishLANG = 0;
  /* Check if we are on spanish */
  const char *buf=QLocale::system().name().toLatin1().data();
  //printf("LOCALE: %s\n", buf);
  if(strstr(buf,"es") != NULL )
    spanishLANG=1;
	/* Check motors */
	for ( uIndexMotor = 0 ; uIndexMotor < NUM_MOTORS ; uIndexMotor++ )                           
	{
		/* Check  vars */
		for (uIndexVar = 0 ; uIndexVar < VARNAMES ; uIndexVar++)                                   
		{
			char filename[100];
			sprintf(filename, "%sMOTOR%d%s", s_path.c_str(),uIndexMotor+1, fileNameVar[uIndexVar]); 
			std::ifstream file(filename);	
			
			/* If file exists */
			if (file)
			{
				/* Set Var is active */
				setCommVarFlag(uIndexVar, true);
				
				/* Read Line by Line */
				std::string sTime; 
				std::string sValue; 
				while (std::getline(file, sTime, ' '))
				{
          getline(file,sValue);
          if(spanishLANG == 1 )
          {
	  				size_t f = sTime.find(".");
		  			sTime.replace(f, 1, ",");
			  		f = sValue.find(".");
				  	sValue.replace(f, 1, ",");
          }
					addValue(uIndexMotor, uIndexVar, atof(sTime.c_str()), atof(sValue.c_str()));
				}//end reading line by line
			}//end if file exists
		}//end for vars
	}//enf for motors

	setupPlot();
}

