#include "qtsecosocket.h"


QtSecoSocket::QtSecoSocket(PlotWindow *q_plot, QLineEdit *queue, QLedIndicator *led_1, QLedIndicator *led_2, QLedIndicator *led_3, QLedIndicator *led_4, QPushButton *send_button, QPushButton *run_button, QPushButton *reset_button,  QPushButton *disconnect_button, CameraWindow *_camera, QObject *parent) : QObject(parent)
{
	socket = new QTcpSocket(this);
	plot = q_plot;

  led1              = led_1;
  led2              = led_2;
  led3              = led_3;
  led4              = led_4;
  queueValue        = queue;
  sendButton        = send_button;
  runButton         = run_button;
	resetButton       = reset_button;
	disconnectButton  = disconnect_button;
  camera            = _camera;

	socketOnFlag = FALSE;
	m_bDataEndFlag = false;
	/* Set timer */                 
	m_TAlife = new QTimer(this);
	connect(m_TAlife, SIGNAL(timeout()), this, SLOT(sendAlife()));

}

bool QtSecoSocket::connectToServer(QString address, unsigned int port, CommunicationDataTab *comm_data_tab)
{

	//Set CommDataTab global 
	commDataTab = comm_data_tab;
	
	//Convert String address to QHostAddress
	QHostAddress host_address;
	host_address.setAddress(address);
	
	//Connect to address and port
	socket->connectToHost(host_address, port);

	//Wait 1 sec for connection
	//If connected, return true otherwise return false
	if (socket->waitForConnected(1000))
	{
		//Connect readings to on_readyRead
		connect(socket, SIGNAL(readyRead()), this, SLOT(on_readyRead()));
		connect(socket, SIGNAL(disconnected()), this, SLOT(socket_disconnected()));

		socketOnFlag = TRUE;
		/* Start ALIFE timer */
		startAlifeTimer();

		return true;
	}
	else
	{
		socketOnFlag = FALSE;
		return false;
	}

}

/*********************************
 *********************************/
		
void QtSecoSocket::disconnectFromServer( void )
{
	//printf("A\n");
	reset();
	
	/* Sttop ALIFE timer */
	stopAlifeTimer();

	//printf("B\n");
	socket->disconnectFromHost();
	//printf("C\n");
}

/*********************************
 *********************************/

void QtSecoSocket::sendData (unsigned char frame_type, unsigned char funct, unsigned char u_motor, float data1, float data2, float data3 )
{
	double timestamp = 0.0; 

	/* Get Time */
	struct timeval start;
	char sendline[MAXLINE];
	
	//Get actual Time
	gettimeofday(&start,0);
	timestamp = (double) start.tv_sec + ((double) start.tv_usec ) * 0.000001;
	//Create frame
	sprintf(sendline,"[%2f:%d:%d:%d:%2.4f:%2f:%2f]", timestamp, frame_type, funct, u_motor, data1, data2, data3);

	//Correct commas by points because of QT in Spanish
	replaceCommaByPoint(sendline);
	//Set end line
	int len = strlen(sendline);
	sendline[len] = '\n';
	sendline[len+1] = '\0';
	
	/* DEBUG */
  //printf("%s\n",sendline);
	/* DEBUG */

	//Send frame in socket
	socket->write(sendline);
	socket->flush();
	//Small delay to prevent frames lost
	usleep(10000);
}

/*********************************
 *********************************/

void QtSecoSocket::replaceCommaByPoint (char* sendline)
{
	for ( unsigned int j = 0 ; j <  strlen(sendline)+1 ; j++)
	{
		if(sendline[j]==',')
			sendline[j] = '.';
	}
}

/*********************************
 *********************************/

void QtSecoSocket::replacePointByComma (char* sendline)
{
	for ( unsigned int j = 0 ; j <  strlen(sendline)+1 ; j++)
	{
		if(sendline[j]=='.')
			sendline[j] = ',';
	}
}

/*********************************
 *********************************/

void QtSecoSocket::reset ( void )
{
	sendData(RESET_TYPE, RESET_SYSTEM_RESET, 0, 0, 0, 0);
	//Extra time to allow the system restart
	usleep(500000);
}

/*********************************
 *********************************/

void QtSecoSocket::run ( NetworkTab *network_tab, MotorTab *motor_tab, ControllerTab *controller_tab, PerturbationTab *perturbation_tab, CommunicationTab *communication_tab  )
{
	/* Start camera */
	//camera->cameraStart("http://138.100.21.254:3010/jpg/1/image.jpg");
	camera->cameraStart(network_tab->getIP(), network_tab->getCameraPort());
	//sleep(2);

	//Get data of files to write on. Need to be divided in to because toLatin is overwritting things
	QByteArray tmpFileArray = network_tab->getFilePath().toLatin1(); 
	QByteArray tmpPrefixArray = network_tab->getFilePrefix().toLatin1(); 
	const char *tempPath = tmpFileArray.data();
	const char *tempPrefix = tmpPrefixArray.data();
	//sprintf(sFilePath, "%s/%s-", tempPath, tempPrefix);
	sFilePath.assign(tempPath);
	sFilePath.append("/");
	sFilePath.append(tempPrefix);
	sFilePath.append("-");

	string fileToDelete("rm -rf ");
	fileToDelete.append(sFilePath);
	fileToDelete.append("MOTOR*");
	//printf("Deleting: %s\n", fileToDelete.c_str());
	system(fileToDelete.c_str());

	/* Reset the boards */	
	reset();

	/* Clean Socket */	
	while(socket->canReadLine())
	{
		//printf("HOLA\n");
		char buf[MAXLINE]="";
		//Read Input Data
		socket->read(&buf[0],MAXLINE);
	}

	//Init Motors
	for (int u_motor = 0 ; u_motor < NUM_MOTORS ; u_motor++)
	{
		//If motor active, init motor
		if ( motor_tab->getMotorStatus(u_motor) )
		{
			//Get PWM freq
			float fMotorPWMFreq = motor_tab->getMotorPWMFreq(u_motor);
			sendData(MOTOR_TYPE, INIT_MOTOR, u_motor, fMotorPWMFreq,0,0);
		}	
	}

	//Activate Current Module
	unsigned char currentFlag = 0;
	for ( int u_motor = 0 ; u_motor < NUM_MOTORS ; u_motor ++ )
	{
		//If motor active, init motor
		if ( motor_tab->getMotorStatus(u_motor) &&
				 communication_tab->getCommunicationParamValue (u_motor, CURRENT) == true)
			{
				currentFlag = 1;
				break;
			}
	}

	if (currentFlag == 1 )
		sendData(MOTOR_TYPE, INIT_CURRENT, 0, 1, 0, 0);

	//Set Controller Sampling rate
	for ( int u_motor = 0 ; u_motor < NUM_MOTORS ; u_motor ++ )
	{
		//If motor active, init motor
		if ( motor_tab->getMotorStatus(u_motor) )
		{
			//Get PWM freq
			float fControllerSamplingFreq = controller_tab->getControllerSamplingFreq(u_motor);
			sendData(CONTROLLER_TYPE, INIT_CONTROLLER, u_motor, fControllerSamplingFreq, 0, 0);
		}
	}

	//Set Controller Params
	unsigned char num_controllers = controller_tab->getNumControllers();
	unsigned char num_params = controller_tab->getNumParams();

	//Check for motors
	for ( int u_motor = 0 ;  u_motor < NUM_MOTORS ; u_motor++ ) 
	{
		//If motor ON
		if ( motor_tab->getMotorStatus(u_motor) )
		{
			for ( int u_controller = 0 ; u_controller < num_controllers; u_controller++ )
			{
				for (int u_param = 0 ; u_param < num_params ; u_param++ )
				{
					float paramValue = controller_tab->getControllerParamValue(u_motor, u_controller, u_param);
					if ( paramValue != 0.0 )
						sendData(CONTROLLER_TYPE, SET_CONTROLLER_PARAMS, u_motor, u_controller, u_param, paramValue);
				}//end param
			}//end controller
			//Tell all parameters have been sent
			sendData(CONTROLLER_TYPE, SEND_CONTROLLER_PARAMS, u_motor, 0, 0, 0);
		}//end if motor
	}//end motor


	//Send Reference
	for ( int u_motor = 0 ; u_motor < NUM_MOTORS ; u_motor++ )
	{
		if ( motor_tab->getMotorStatus(u_motor) )
		{
			unsigned int 	referenceType = controller_tab->getControllerRefType(u_motor);
			float 				referenceValue1 = controller_tab->getControllerRefValue1(u_motor);
			float 				referenceValue2 = controller_tab->getControllerRefValue2(u_motor);
			
			if ( referenceType == (EXP - 1) )//Problems in numberes with TRAP and EXP because they do have 4 and 3 parameters. Check motortab.h
			{
				sendData(CONTROLLER_TYPE, SET_REF, u_motor, EXP, referenceValue1, referenceValue2);
				float 				referenceValue3 = controller_tab->getControllerRefValue3(u_motor);
				sendData(CONTROLLER_TYPE, SET_REF, u_motor, EXP2, referenceValue3, 0);
			}
			/* Do for all except EXP */
			else
			{
				sendData(CONTROLLER_TYPE, SET_REF, u_motor, referenceType, referenceValue1, referenceValue2);
			
				if ( controller_tab->getControllerRefType(u_motor) == TRAPEZOIDAL )
				{
					float 				referenceValue3 = controller_tab->getControllerRefValue3(u_motor);
					float 				referenceValue4 = controller_tab->getControllerRefValue4(u_motor);
					sendData(CONTROLLER_TYPE, SET_REF, u_motor, TRAPEZOIDAL2, referenceValue3, referenceValue4);
				}
			}	
		}
	}//end for motor

	//Send Perturbation
	for ( int u_motor = 0 ; u_motor < NUM_MOTORS ; u_motor++ )
	{
		if ( motor_tab->getMotorStatus(u_motor) )
		{
			unsigned int 	perturbationType = perturbation_tab->getPerturbationRefType(u_motor);
			if ( perturbationType != 0 )
			{
				float 				perturbationValue1 = perturbation_tab->getPerturbationRefValue1(u_motor);
				float 				perturbationValue2 = perturbation_tab->getPerturbationRefValue2(u_motor);

				if ( perturbationType == (EXP -1 ) )
				{
					sendData(CONTROLLER_TYPE, SET_PERTURBATION, u_motor, EXP, perturbationValue1, perturbationValue2);
					float 				perturbationValue3 = perturbation_tab->getPerturbationRefValue3(u_motor);
					sendData(CONTROLLER_TYPE, SET_PERTURBATION, u_motor, EXP2, perturbationValue3, 0);
				}
				/* Do for all excep EXP */
				else
				{
					sendData(CONTROLLER_TYPE, SET_PERTURBATION, u_motor, perturbationType, perturbationValue1, perturbationValue2);
					
					if ( perturbation_tab->getPerturbationRefType(u_motor) == TRAPEZOIDAL )
					{
						float 				perturbationValue3 = perturbation_tab->getPerturbationRefValue3(u_motor);
						float 				perturbationValue4 = perturbation_tab->getPerturbationRefValue4(u_motor);
						sendData(CONTROLLER_TYPE, SET_PERTURBATION, u_motor, TRAPEZOIDAL2, perturbationValue3, perturbationValue4);
					}
				}
			}//end NO PERTURBATION	
		}
	}//end for motor
	
	//Set Comm Params
	unsigned char num_comm_params = communication_tab->getNumParams();
	for ( int u_motor = 0 ; u_motor < NUM_MOTORS ; u_motor++ )
	{
		if ( motor_tab->getMotorStatus(u_motor) )
		{
			for ( int u_param = 0 ; u_param < num_comm_params ; u_param++ )
			{
				unsigned int paramValue = communication_tab->getCommunicationParamValue(u_motor, u_param);
				if ( paramValue != 0.0)
					sendData(COMM_TYPE, SET_COMM_PARAMS, u_motor, u_param, paramValue, 0);
			}
			//Tell all parameters have been sent
			sendData(COMM_TYPE, SEND_COMM_PARAMS, u_motor, 0, 0, 0);
		}
	}

	//Config Comm freq
	for ( int u_motor = 0 ; u_motor < NUM_MOTORS ; u_motor++ )
	{
		if ( motor_tab->getMotorStatus(u_motor) )
		{
			unsigned int txFreq = communication_tab->getCommunicationTransmissionFreq(u_motor);
			float txTimeLimit = communication_tab->getCommunicationTimeLimit(u_motor);
			//printf("Time Limit: %2.2f\n", txTimeLimit);
			sendData(COMM_TYPE, COMM_CONFIG, 0, txFreq, txTimeLimit, 0 );
		}
	}
	
	
	//Init Send Comm 
	for ( int u_motor = 0 ; u_motor < NUM_MOTORS ; u_motor++ )
	{
		if ( motor_tab->getMotorStatus(u_motor) )
		{
			sendData(COMM_TYPE, INIT_SEND_COMM, 0, 0, 0, 0 );
		}
	}
	
	//usleep(100000);	
	//Start Controller
	for ( int u_motor = 0 ; u_motor < NUM_MOTORS ; u_motor++ )
	{
		if ( motor_tab->getMotorStatus(u_motor) )
		{
			unsigned char u_controller_type = controller_tab->getControllerType(u_motor);
			unsigned char u_control_var	    = controller_tab->getControllerControlVar(u_motor);
			/* Set names for legen on plot */
			plot->setPrintNames(u_control_var);
			sendData(CONTROLLER_TYPE, START_CONTROLLER, u_motor, u_controller_type, u_control_var, 0 );
		}
	}
}

/*********************************
 *********************************/

void QtSecoSocket::on_readyRead()
{
	while(socket->canReadLine())
	{
		char buf[MAXLINE]="";
		//Read Input Data
		socket->read(&buf[0],MAXLINE);
		/* DEBUG */
		//Send data to logConsole
    //commDataTab->appendLine(buf);			
    //printf("%s", buf);
    //puts(buf);
		/* DEBUG */
		
		/* If is a headear, with filenmae, which includes MOTOR, 
		 * set the fileName name */
		if ( strstr(buf,"MOTOR") != NULL )
		{
			sFileName.assign(buf);
			sFileName[sFileName.size()-1]='\0';
      led1->setChecked(false);
      led2->setChecked(false);
      led3->setChecked(false);
      led4->setChecked(true);
		}
		/* If is end of tx, plot */
		else if ( strstr(buf,"EndOfTx") != NULL )
		{
			//printf("1\n");
      disconnectFromServer();
			//printf("2\n");
      //Set Led Red
      led1->setChecked(true);
      led2->setChecked(false);
      led3->setChecked(false);
      led4->setChecked(false);
      //Stop Camera
      camera->cameraStop();
			plot->loadPlotsFromSocket(sFilePath);
			setDataEnd(true);
      //Enable Send buttom
      sendButton->setEnabled(true);
      resetButton->setEnabled(false);
      disconnectButton->setEnabled(false);
			printf("Disconnected\n");
		}
    else if (strstr(buf,"Queue:") != NULL)
    {
			//usleep(500000);
      char tmp[4];
      tmp[0]=buf[6];
      tmp[1]=buf[7];
      tmp[2]=buf[8];
      tmp[3]='\n';
      int tmpQueue=atoi(tmp);
      //Get position of queue
			//printf("Queue2: %d\n", tmpQueue);
      queueValue->clear();
      QString qtmp = tr("%3").arg(tmpQueue);
      queueValue->insert(qtmp);

      if (tmpQueue == 0)
      {
        led1->setChecked(false);
        led2->setChecked(false);
        led3->setChecked(true);
        led4->setChecked(false);
        runButton->click();
      }
      else if (tmpQueue == 1)
      {
        led1->setChecked(false);
        led2->setChecked(true);
        led3->setChecked(false);
        led4->setChecked(false);
      }
    }
		/* else, parse data to file */
		else
		{
			//Print data to File
			parseToFile(buf);
		}
	}
}

/*********************************
 *********************************/

void QtSecoSocket::parseToFile ( char* buf )
{
	FILE *f1;
	string tmp;
	tmp.assign(sFilePath);
	tmp.append(sFileName);
	
	/* DEBUG */
	//printf("%s", buf);
	//printf("WRITING TO FILE:%s -- %d\n", tmp.c_str(), strlen(buf));
	//fflush(stdout);
	/* DEBUG */
	
	
	f1 = fopen(tmp.c_str(), "a");
	fprintf(f1, "%s", buf);
	fclose(f1);

}

/*********************************
 *********************************/

void QtSecoSocket::parseToPlot ( char* buf )
{
	//FILE *f1;
	//char sFilename[100];

	
	char strTimestamp[30];
	char strTime[20];
	char strMotor[2];
	char strVar[5];
	char strValue[20];

	unsigned int count = 1;
	unsigned int subCount = 0;
	//Get timestamp
	while ( buf[count] != ':')
		strTimestamp[subCount++] = buf[count++];

	count++;
	strTimestamp[subCount]='\0';
  //replacePointByComma(strTimestamp);

	//Get time
	subCount=0;
	while ( buf[count] != ':')
		strTime[subCount++] = buf[count++];

	count++;
	strTime[subCount]='\0';
  //replacePointByComma(strTime);

	//Get motor
	subCount=0;
	while ( buf[count] != ':')
		strMotor[subCount++] = buf[count++];

	count++;
	strMotor[subCount]='\0';
  //replacePointByComma(strMotor);
	
	//Get var
	subCount=0;
	while ( buf[count] != ':')
		strVar[subCount++] = buf[count++];

	count++;
	strVar[subCount]='\0';
  //replacePointByComma(strVar);

	//Get value
	subCount=0;
	while ( buf[count] != ']')
		strValue[subCount++] = buf[count++];

	strValue[subCount]='\0';
  //replacePointByComma(strValue);

	/* DEBUG */ 
  //printf("TIMESTAMP: %s\n", timestamp);
  printf("TIME: %s\n", strTime);
  fflush(stdout);
  //printf("MOTOR: %s\n", motor);
  //printf("VAR: %s\n", var);
  //printf("VALUE: %s\n", value);
	/* DEBUG */
	
	float 				fTime 	= atof(strTime);
	unsigned char uMotor 	= atoll(strMotor);
	float 				fValue 	= atof(strValue);
	unsigned char uVar		= 0;

	if ( strcmp(strVar, "POS") == 0)
		uVar = POS;
	else if ( strcmp(strVar, "VEL") == 0)
		uVar = SPEED;
	else if ( strcmp(strVar, "CUR") == 0)
		uVar = CURRENT;
	else if ( strcmp(strVar, "REF") == 0)
		uVar = REF;
	else if ( strcmp(strVar, "ERR") == 0)
		uVar = ERROR;
	else if ( strcmp(strVar, "U  ") == 0)
		uVar = U;

	plot->addValue(uMotor, uVar, fTime, fValue);

}

void QtSecoSocket::startAlifeTimer ( void )
{
	m_TAlife->start(1000);
}

void QtSecoSocket::stopAlifeTimer ( void )
{
	m_TAlife->stop();
}

void QtSecoSocket::sendAlife ( void )
{
	sendData(RESET_TYPE, RESET_ALIFE, 0, 0, 0, 0);
}


void QtSecoSocket::socket_disconnected()
{
	//printf("Disconnected\n");
	socketOnFlag = FALSE;
}

unsigned char QtSecoSocket::getSocketStatus()
{
	return socketOnFlag;
}
		
void 	QtSecoSocket::setDataEnd		(	bool flag)
{
	m_bDataEndFlag = flag;
}

bool	QtSecoSocket::getDataEnd		( void )
{
	return m_bDataEndFlag;
}
